'<ADbasic Header, Headerversion 001.001>
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--GOLD
'
'###############################################################
'#                        ADwGCNT.inc                          #
'#              INCLUDE-file for ADwin-GOLD-system             #
'###############################################################
'#                                                             #
'# Contents: Include functions for ADwin-GOLD-CO1              #
'#                                                             #
'#  22.06.1999  HM   creation                                  #
'#  21.03.2000  JK   extracted : INC PAR_14  ,  multiple reads #
'#                   and latches at CNT_READ (marked with OLD1)#
'#  10.04.2000  JK   header of this file (the first 13 lines)  #
'#                   again inserted, to avoid problems when    #
'#                   editing this file with the ADbasic editor #
'#  28.11.2000  JK   OLD1 comments deleted ; improved comment  #
'#                   for CNT_READLATCH                         #
'#  16.11.2001  JK   1st line : ADBASIC-SYSTEM_FILE...         #
'#  14.08.2002  JK   cntnr   ->  (cntnr)                       #
'#  02.06.2003  MF   CNT_GETSTATUS,CNT_RESETSTATUS,CNT_SE_DIFF #
'#  14.05.2007  MF   designed for ADbasic 5                    #
'#  19.03.2009  MF   '''+                                      #
'#  17.09.2009  MF   '''+ CNT_Mode                             #
'#                                                             #
'# Attention:                                                  #
'#   - Do not copy the first source line (containing           #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !              #
'#     Otherwise the Syntax color may be not correct.          #
'#                                                             #
'###############################################################


' ************************************************************
''' sets one or more counters to zero, according to the bit pattern in pattern.
''' pattern:
'''+       Bit = 0: no influence.
'''+       Bit = 1: set counter to zero.
SUB CNT_Clear(pattern) 
  ' clear all counters selected with a bit 1
  POKE(20400310h,pattern)
ENDSUB

''' disables or enables the counters set by pattern, to count incoming impulses
'''pattern:
'''+         Bit = 0: stop counter.
'''+         Bit = 1: enable counter.
''' **********************************************************
SUB CNT_Enable(pattern) 
  ' enable all counters selected with a bit 1
  POKE(20400300h,pattern)
ENDSUB


''' defines the operating mode for all counters (depending on Cnt_Set) according to the given bit pattern.
'''+ Bit value in pattern
''' pattern:
'''+  CNT_SET=0
'''+             pattern 00 = 4-edge evaluation    
'''+             pattern 11 = Reference clock 20 MHz
'''+  CNT_SET=1
'''+             pattern 00 = Clock and direction input
'''+             pattern 11 = Reference clock 5 MHz
''' **************************************************************
SUB CNT_Mode(pattern) 
  ' Select Counter-mode  (0 = CLK external, 1 = CLK internal(PWM))
  POKE(20400340h,pattern)
ENDSUB


' **************************************************************
''' defines the operating mode for all counters (depending on Cnt_Mode) according to the given bit pattern.
'''+ Bit value in pattern
''' pattern:
'''+  CNT_MODE=0
'''+             pattern 00 = 4-edge evaluation    
'''+             pattern 11 = Clock and direction input
'''+  CNT_MODE=1
'''+             pattern 00 = Reference clock 20 MHz
'''+             pattern 11 = Reference clock 5 MHz
''' **************************************************************
SUB CNT_Set(pattern) 
  ' Set Counter   (0 = 4fach, 1 = CLK/DIR)
  ' Set PWM-Counter Frequency (0 = 20 MHz, 1 = 5 MHz)
  POKE(20400350h,pattern)
ENDSUB


' ************************************************************
'''sets the function of the CLR/LATCH input of one or more counters.
'''pattern:
'''+        Bit = 0: Set CLR-mode.
'''+        Bit = 1: Set LATCH-mode.
SUB CNT_InputMode(pattern) 
  ' Set Counterinputline   (0 = Clear counter, 1 = latch counter)
  POKE(20400330h,pattern)
ENDSUB


' ************************************************************
'''transfers the current counter values of one or more counters on the specified module into the respective latch register(s) (= to latch).
'''pattern:
'''+        Bit = 0: no function.
'''+        Bit = 1: transfer counter values into Latch A .
SUB CNT_Latch(pattern) 
  ' latch all counters selected with a Bit 1
  POKE(20400320h,pattern)
ENDSUB


' ***********************************************************
'''copies the current counter value into Latch A and returns it as return value.
'''cntno: Counter number: 1..4
FUNCTION CNT_Read(cnt_no) AS Long
  ' performs a software-latch and reads the selected counter
  ' ATTENTION : this function may NOT be used, if the latch is
  ' done by an external signal (e.g. period-time or pulse 
  ' width measurements) !
  DIM cadr AS Long
  DIM cntsel AS Long
  cntsel = SHIFT_LEFT(1,(cnt_no-1))
  POKE(20400320h, cntsel)
  cadr = 204001F4h + (cnt_no) * 16	' parentheses because i + 1
  ' could be passed as cntnr
  CNT_READ = PEEK(cadr)
  ' (we do not need multiple reads, because for software
  ' latch the synchronisation is done by hardware.)    
ENDFUNCTION


' ***********************************************************
''' returns the value of a counter previously stored in Latch A.
''' cnt_no: Counter number: 1..4
FUNCTION CNT_ReadLatch(cnt_no) AS Long
  ' reads the latch register triggerd by software-latch or 
  ' signal-rise (of an external signal)
  DIM cadr AS Long
  DIM ih AS Long
  cadr = 204001F4h + (cnt_no) * 16	' parentheses because i + 1
  ' could be passed as cntnr
  CNT_READLATCH = PEEK(cadr)

  ' case a : the latch register is triggered by a software- latch :
  ' ---------------------------------------------------------------
  ' In this case, there is no need for multiple reads, because 
  ' the synchronisation is done by hardware (see also the 
  ' comments at CNT_READ()). The following instructions are 
  ' not necessary, but they also do not harm (except wasting
  ' time). Therefore they are a potential for speed optimi-
  ' sations.

  ' case b : the latch register is triggered by an external signal :
  ' ----------------------------------------------------------------
  ' Check, if there was a (from an external signal triggered)
  ' LATCH just in the moment, when we read the latch 
  ' (This is neccessary because this case is not
  ' handled by the hardware).
  ' If we have this case, the second reading of the latch 
  ' register differs from the first reading and therefore 
  ' one of the readings may be corrupt. Read a third time to 
  ' be sure to have a valid reading. 
  '   The measured time for these three readings is 25 
  '   cycles = 625 ns   => the (period) time between two
  '   successive (external signal triggered) LATCHs may 
  '   NOT be shorter than this limit.
  '   => valid measurements (using this method with LATCH
  '   done by external signal) cannot be guaranteed for 
  '   smaller LATCH (period) times than 625 ns. (Longtime 
  '   tests were done and o.k. with 664 ns). To have a  
  '   safety margin for future changes in the ADbasic  
  '   compiler, our statement to the customer is the  
  '   limit of 1000 ns. 

  ih = PEEK(cadr)
  IF (ih <> CNT_READLATCH) THEN
    CNT_READLATCH = PEEK(cadr)
  ENDIF
ENDFUNCTION


' ***********************************************************
''' returns the value of a counter previously stored in Latch B.
''' cnt_no: Counter number: 1..4
FUNCTION CNT_ReadFLatch(cnt_no) AS Long
  ' reads the latch register triggerd by signal-falling edge
  DIM cadr AS Long
  DIM ih AS Long
  cadr = 204001F8h + (cnt_no) * 16	' parentheses because i + 1
  ' could be passed as cntnr
  CNT_READFLATCH = PEEK(cadr)
  ' (please see comments to READLATCH)
  ih = PEEK(cadr)
  IF (ih <> CNT_READFLATCH) THEN
    CNT_READFLATCH = PEEK(cadr)
  ENDIF
ENDFUNCTION


'***************************************************************
''' reads the status of a counter input 
'''+   Bit 4 CLR-/LATCH-input (signal is not changing state) Counter 1
'''+   Bit 5 CLR-/LATCH-input (signal is not changing state) Counter 2
'''+   Bit 6 CLR-/LATCH-input (signal is not changing state) Counter 3
'''+   Bit 7 CLR-/LATCH-input (signal is not changing state) Counter 4
'''+   Bit 16 Signal A (signal is not changing states) Counter 1
'''+   Bit 17 Signal B (signal is not changing states) Counter 1
'''+   Bit 18 Signal A (signal is not changing states) Counter 2
'''+   Bit 19 Signal B (signal is not changing states) Counter 2
'''+   Bit 20 Signal A (signal is not changing states) Counter 3
'''+   Bit 21 Signal B (signal is not changing states) Counter 3
'''+   Bit 22 Signal A (signal is not changing states) Counter 4
'''+   Bit 23 Signal B (signal is not changing states) Counter 4
'''+   Bit 24 Correlation error (signals A and B are identical, they are not phase-shifted by approx. 90)
'''+   Counter 1
'''+   Bit 25 Line error (cable not connected or the line is broken)
'''+   Counter 1
'''+   Bit 26 Correlation error (signals A and B are identical, they are not phase-shifted by approx. 90)
'''+   Counter 2
'''+   Bit 27 Line error (cable not connected or the line is broken)
'''+   Counter 2
'''+   Bit 28 Correlation error (signals A and B are identical, they are not phase-shifted by approx. 90)
'''+   Counter 3
'''+   Bit 29 Line error (cable not connected or the line is broken)
'''+   Counter 3
'''+   Bit 30 Correlation error (signals A and B are identical, they are not phase-shifted by approx. 90)
'''+   Counter 4
'''+   Bit 31 Line error (cable not connected or the line is broken)
'''+   Counter 4
'''***************************************************************
FUNCTION CNT_GetStatus() AS Long
  CNT_GETSTATUS = PEEK(20400370h)           
ENDFUNCTION


'**************************************************************
' resets the error bits of the status register 
'**************************************************************
SUB CNT_ResetStatus() AS Long
  POKE(20400370h,0)
ENDSUB


'***************************************************************
''' Selects single-ended (s.e.) or differential (diff.) inputs'
''' pattern:
'''+ select: s.e. -> 0 (default after power-up),  diff. -> 1 
'''+ counter 1,2 selected with Bit 0
'''+ counter 3,4 selected with Bit 1
'''*************************************************************
SUB CNT_Se_Diff(pattern) as Long
  POKE(20400304h,pattern)
ENDSUB

