# check the requested options
AC_DEFUN([ADWIN_CONFIG], [
	AC_MSG_CHECKING([which directory contains the boot rcX.d directories])
	for dir in /etc/rc.d/ /etc/ /etc/init.d/ ; do
	  if test -d "$dir/rc3.d/"; then
	    RCX_D_CONTAINING_DIR=$dir
	    break
	  fi
	done
	if test "$RCX_D_CONTAINING_DIR"; then
	  AC_MSG_RESULT([$RCX_D_CONTAINING_DIR])
	else
	  AC_MSG_RESULT([none found])
	fi
	AC_SUBST(RCX_D_CONTAINING_DIR)

	AC_MSG_CHECKING([kernel version])
	kernel_version=[`uname -r | sed 's/\([^\.]*\.[^\.]*\).*/\1/ '`]
	if test "$kernel_version" ; then
	  AC_MSG_RESULT([$kernel_version])
	else
	  AC_MSG_RESULT([could not determine kernel version])
	fi

	AC_MSG_CHECKING(if embedded version is requested)
	AC_ARG_ENABLE(embedded,
	        AC_HELP_STRING([--enable-embedded],
	                [build for embedded system (default=disabled)]),
	        [ ADWIN_EMBEDDED="$enableval" ],
		[ ADWIN_EMBEDDED="no" ]
	)
	if test "$ADWIN_EMBEDDED" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([ADWIN_EMBEDDED], [1], [Build for embedded system (IXP)])
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if debug output is requested)
	AC_ARG_ENABLE(debug,
	        AC_HELP_STRING([--enable-debug],
	                [enable DEBUG support (default=disabled)]),
	        [ ADWIN_DEBUG="$enableval" ],
		[ ADWIN_DEBUG="no" ]
	)
	if test "$ADWIN_DEBUG" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([ADWIN_DEBUG], [2], [Set debug level])
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if link support is requested)
	AC_ARG_ENABLE(link,
	        AC_HELP_STRING([--disable-link],
	                [disable link support (default=enabled)]),
	        [ adwin_with_link="$enableval" ],
		[ adwin_with_link="no" ]
	)
	if test "$adwin_with_link" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([WITH_LINK], [], [Define to enable link support])
		ADWIN_LINK_O=adwin_link.o
		AC_SUBST(ADWIN_LINK_O)
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if ethernet support is requested)
	AC_ARG_ENABLE(ethernet,
	        AC_HELP_STRING([--disable-ethernet],
	                [disable ethernet support (default=enabled)]),
	        [ adwin_with_net="$enableval" ],
		[ adwin_with_net="yes" ]
	)
	if test "$adwin_with_net" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([WITH_ETHERNET], [], [Define to enable ethernet support])
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if USB support is requested)
	AC_ARG_ENABLE(usb,
	        AC_HELP_STRING([--enable-usb],
	                [enable USB support (default=disabled)]),
	        [ adwin_with_usb="$enableval" ],
		[ adwin_with_usb="no" ]
	)
	if test "$adwin_with_usb" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([WITH_USB], [], [Define to enable USB support])
		echo "USB is currently not supported on Linux!"
		exit 1;
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if PCMCIA support is requested)
	AC_ARG_ENABLE(pcmcia,
	        AC_HELP_STRING([--enable-pcmcia],
	                [enable PCMCIA support (default=disabled)]),
	        [ ADWIN_WITH_PCMCIA="$enableval" ],
		[ ADWIN_WITH_PCMCIA="no" ]
	)
	if test "$ADWIN_WITH_PCMCIA" = "yes"; then
	        AC_MSG_RESULT(yes)
		AC_DEFINE([WITH_PCMCIA], [], [Define to enable PCMCIA support])
		if test "$adwin_with_link" = "no"; then
			echo "PCMCIA support requires link support!"
			exit 1;
		fi
		ADWIN_CS_O=adwin_cs.o
		AC_SUBST(ADWIN_CS_O)
	else
	        AC_MSG_RESULT(no)
	fi

	AC_MSG_CHECKING(if kernel module should be built)
	AC_ARG_ENABLE(kernel_module,
	        AC_HELP_STRING([--disable-kernel-module],
	                [disable kernel module support (default=enabled)]),
	        [ kernel_module="$enableval" ],
		[ kernel_module="no" ]
	)
	if test "$kernel_module" = "yes"; then
	        AC_MSG_RESULT(yes)
		if test "$kernel_version" != 2.4 ; then 
			echo "Kernel module support is only available for kernel 2.4!"
			exit 1;
		fi
		AC_MSG_CHECKING(if kernel supports proc-fs)
		if test ! -f /proc/ioports ; then
		        AC_MSG_RESULT(no)
			echo "/proc/ioports does not exist!"
			echo "adwin-lib requires proc-fs at location /proc"
			exit 1;
		fi	
	        AC_MSG_RESULT(yes)
		AC_DEFINE([KERNEL_MODULE], [], [Define to use kernel module])
		AC_DEFINE([CONFIG_FILE], ["/proc/adwin/devices"], [Store config in kernel])	
		source /lib/modules/`uname -r`/build/.config
		if test "$CONFIG_SMP" = "y" ; then
			SMP_FLAG=-D__SMP__
		fi
		if test "$CONFIG_MODVERSIONS" = "y" ; then
			AC_DEFINE([ADWIN_MODVERSIONS],[], [Compile module version information])
		fi
		AC_SUBST(SMP_FLAG)
	else
	        AC_MSG_RESULT(no)
		if test "$adwin_with_link" = "yes" ; then 
			echo "Link support requires kernel module!"
			exit 1;
		fi
		AC_DEFINE([CONFIG_FILE], ["/etc/adwin/devicelist"], [Use config in regular file in /etc/])
	fi
])


dnl
dnl Checks to see if the sockaddr struct has the 4.4 BSD sa_len member
dnl
dnl usage:
dnl
dnl	AC_LBL_SOCKADDR_SA_LEN
dnl
dnl results:
dnl
dnl	HAVE_SOCKADDR_SA_LEN (defined)
dnl
AC_DEFUN([AC_LBL_SOCKADDR_SA_LEN],
    [AC_MSG_CHECKING(if sockaddr struct has sa_len member)
    AC_CACHE_VAL(ac_cv_lbl_sockaddr_has_sa_len,
	AC_TRY_COMPILE([
#	include <sys/types.h>
#	include <sys/socket.h>],
	[u_int i = sizeof(((struct sockaddr *)0)->sa_len)],
	ac_cv_lbl_sockaddr_has_sa_len=yes,
	ac_cv_lbl_sockaddr_has_sa_len=no))
    AC_MSG_RESULT($ac_cv_lbl_sockaddr_has_sa_len)
    if test $ac_cv_lbl_sockaddr_has_sa_len = yes ; then
	    AC_DEFINE(HAVE_SOCKADDR_SA_LEN,1,[if struct sockaddr has sa_len])
    fi])

