#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stddef.h>

#include "libadwin_compat.h"
#include "libadwin/errno.h"
#include "libadwin.h"

//**************************************************************************
// DLL-Function simulation
// These functions are mainly used by LabView, since the LabView VIs
// for Windows, MAC and Linux are the same. These functions are wrappers
// which map the Windows DLL-functions to linux the functions.
//**************************************************************************

// Boot
int32_t ADboot(const char *Dateiname, int16_t Linkadresse, int32_t iboardsize, int16_t msgbox)
{
    Set_DeviceNo (Linkadresse);
    return adwin_boot(Dateiname, iboardsize, Linkadresse);
}
int32_t e_ADboot(const char *Dateiname, int16_t Linkadresse, int32_t iboardsize, int16_t msgbox, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (Linkadresse);
    tmp = adwin_boot(Dateiname, iboardsize, Linkadresse);
    *error = Get_Last_Error();
    return tmp;
}


int32_t ADGetErrorCode(void)
{
    int32_t tmp_errno;

    tmp_errno = adwin_errno;
    adwin_errno = 0; //damit es kompatibel zu Windows-DLL ist;
    return tmp_errno;
}

int32_t ADGetErrorText(int32_t ErrCode, char *pText, int32_t len)
{
    adwin_strerror_r(ErrCode, pText, len);
    return 0;
}

// Load_Process
int16_t ADBload(const char *dateiname, int16_t Linkaddresse, int16_t msgbox)
{
    Set_DeviceNo (Linkaddresse);
    return adwin_process_load(dateiname, Linkaddresse);
}
int16_t e_ADBload(const char *dateiname, int16_t Linkaddresse, int16_t msgbox,
                  int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo(Linkaddresse);
    tmp = adwin_process_load(dateiname, Linkaddresse);
    *error = Get_Last_Error();
    return tmp;
}

// Start_process
int16_t ADB_Start(int16_t np, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_process_start(np, nadr);
}
int16_t e_ADB_Start(int16_t np, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo(nadr);
    tmp = adwin_process_start(np, nadr);
    *error = Get_Last_Error();
    return tmp;
}

extern int16_t Start(int16_t np, int16_t nadr)   //Start von Standardprozessen
{
    adwin_errno = k_function_not_allowed;
    return 255;
}

extern int16_t Stop(int16_t np, int16_t nadr)   //Stop von Standardprozessen
{
    adwin_errno = k_function_not_allowed;
    return 255;
}

extern int16_t Get_List(int16_t *tarr, int16_t typ, int16_t nadc, int16_t proc, int32_t il, int16_t nadr)
{
    // Standardprozesse werden nicht unterstuetzt
    adwin_errno = k_function_not_allowed;
    return 255;
}

extern int16_t Set_List(int16_t *tarr, int16_t typ, int16_t nadc, int32_t il, int16_t nadr)
{
    // Standardprozesse werden nicht unterstuetzt
    adwin_errno = k_function_not_allowed;
    return 255;
}

// Stop_Process
int16_t ADB_Stop(int16_t np, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_process_stop(np, nadr);
}
int16_t e_ADB_Stop(int16_t np, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_process_stop(np, nadr);
    *error = Get_Last_Error();
    return tmp;
}

int16_t ADSetLanguage(int16_t Lang)
{
    adwin_errno = k_function_not_allowed;
    return 255;
}

// Set_Par
int16_t Set_ADBPar(int16_t np, int32_t iw, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_set_par(np, iw, nadr);
}
int16_t e_Set_ADBPar(int16_t np, int32_t iw, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_set_par(np, iw, nadr);
    *error = Get_Last_Error();
    return tmp;
}

// Set_FPar
int16_t Set_ADBFPar(int16_t np, float fw, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_set_fpar(np, fw, nadr);
}
int16_t e_Set_ADBFPar(int16_t np, float fw, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_set_fpar(np, fw, nadr);
    *error = Get_Last_Error();
    return tmp;
}

//Get_Par
int32_t Get_ADBPar (int16_t np, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_get_par(np, nadr);
}
int32_t e_Get_ADBPar (int16_t np, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_get_par(np, nadr);
    *error = Get_Last_Error();
    return tmp;
}

// Get_FPar
float Get_ADBFPar(int16_t np, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_get_fpar(np, nadr);
}
float e_Get_ADBFPar(int16_t np, int16_t nadr, int32_t *error)
{
    float tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_get_fpar(np, nadr);
    *error = Get_Last_Error();
    return tmp;
}

extern int16_t   SaveFast(const char *dateiname, int16_t Datanr,
                          int32_t startindex, int32_t anzahldaten,
                          int16_t anhaengen, int16_t linkaddresse)
{
    Set_DeviceNo (linkaddresse);
    return adwin_data2file((char *) dateiname, Datanr, startindex,
                           anzahldaten, anhaengen, linkaddresse);
}

extern int16_t e_SaveFast(const char *dateiname, int16_t Datanr,
                          int32_t startindex, int32_t anzahldaten,
                          int16_t anhaengen, int16_t linkaddresse,
                          int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo(linkaddresse);
    tmp = adwin_data2file((char *) dateiname, Datanr, startindex,
                          anzahldaten, anhaengen, linkaddresse);
    *error = Get_Last_Error();
    return tmp;
}

// Get_Data
extern int16_t Get_Data(void *dest, adwin_data_type_t type, int16_t data_no,
                        int32_t start_index, int32_t count, int16_t device_no)
{
    Set_DeviceNo(device_no);
    return _Get_Data(data_no, dest, type, start_index, count, device_no, 0);
}
extern int16_t e_Get_Data(void *dest, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no,
                          int32_t *error)
{
    int32_t tmp;
    Set_DeviceNo(device_no);
    tmp = _Get_Data(data_no, dest, type, start_index, count, device_no, 0);
    *error = Get_Last_Error();
    return tmp;
}

// Set_Data
extern int16_t Set_Data(void *src, adwin_data_type_t type, int16_t data_no,
                        int32_t start_index, int32_t count, int16_t device_no)
{
    Set_DeviceNo(device_no);
    return _Set_Data(data_no, src, type, start_index, count, device_no, 0);
}
extern int16_t e_Set_Data(void *src, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no,
                          int32_t *error)
{
    int32_t tmp;
    Set_DeviceNo(device_no);
    tmp = _Set_Data(data_no, src, type, start_index, count, device_no, 0);
    *error = Get_Last_Error();
    return tmp;
}

// Get_Fifo
extern int16_t   Get_Fifo(void *dest, adwin_data_type_t type, int16_t fifo_no,
                          int32_t count, int16_t device_no)
{
    Set_DeviceNo(device_no);
    return _Get_Fifo(fifo_no, dest, type, count, device_no);
}
extern int16_t e_Get_Fifo(void *dest, adwin_data_type_t type, int16_t fifo_no,
                          int32_t count, int16_t device_no, int32_t *error)
{
    int32_t tmp;
    Set_DeviceNo(device_no);
    tmp = _Get_Fifo(fifo_no, dest, type, count, device_no);
    *error = Get_Last_Error();
    return tmp;
}

// Set_Fifo
extern int16_t   Set_Fifo(void *dest, adwin_data_type_t type, int16_t fifo_no,
                          int32_t count, int16_t device_no)
{
    Set_DeviceNo(device_no);
    return _Set_Fifo(fifo_no, dest, type, count, device_no);
}
extern int16_t e_Set_Fifo(void *dest, adwin_data_type_t type, int16_t fifo_no,
                          int32_t count, int16_t device_no, int32_t *error)
{
    int32_t tmp;
    Set_DeviceNo(device_no);
    tmp = _Set_Fifo(fifo_no, dest, type, count, device_no);
    *error = Get_Last_Error();
    return tmp;
}

// Fifo_Full
int32_t Get_Fifo_Count (int16_t nadc, int16_t nadr )
{
    Set_DeviceNo(nadr);
    return adwin_fifo_full(nadc, nadr);
}

int32_t e_Get_Fifo_Count (int16_t nadc, int16_t nadr, int32_t *error)    // Fifo_Full
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_fifo_full(nadc, nadr);
    *error = Get_Last_Error();
    return tmp;
}

int32_t Get_Fifo_Empty (int16_t nadc, int16_t nadr)   // Fifo_Empty
{
    Set_DeviceNo (nadr);
    return adwin_fifo_empty(nadc, nadr);
}

int32_t e_Get_Fifo_Empty (int16_t nadc, int16_t nadr, int32_t *error)   // Fifo_Empty
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_fifo_empty(nadc, nadr);
    *error = Get_Last_Error();
    return tmp;
}

int16_t Clear_Fifo (int16_t nadc, int16_t nadr )   //Fifo_Clear
{
    Set_DeviceNo (nadr);
    return adwin_fifo_clear(nadc, nadr);
}

int16_t e_Clear_Fifo(int16_t nadc, int16_t nadr, int32_t *error)    //Fifo_Clear
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_fifo_clear(nadc, nadr);
    *error = Get_Last_Error();
    return tmp;
}

uint16_t Get_ADC(int16_t nadc, int16_t nadr)   //ADC
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int16_t Set_DAC(int16_t ndac, int16_t iw, int16_t nadr)    //SetDAC
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int32_t Get_Digin(int16_t nadr)   //DigIn
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int16_t Set_Digout(int16_t iw, int16_t nadr)    //SetDigOut
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int32_t Get_Digout(int16_t nadr)
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int16_t AD_Workload(int32_t Priority, int16_t Linkaddresse)    //Workload
{
    Set_DeviceNo (Linkaddresse);
    return adwin_workload(Priority, Linkaddresse);
}
int16_t e_AD_Workload (int32_t Priority, int16_t Linkaddresse, int32_t *error)   //Workload
{
    int32_t tmp;

    Set_DeviceNo (Linkaddresse);
    tmp = adwin_workload(Priority, Linkaddresse);
    *error = Get_Last_Error();
    return tmp;
}
int16_t ADTest_Version (int16_t Linkaddresse, int16_t msgbox)  // Test_Version
{
    int tmp_ret;
    Set_DeviceNo (Linkaddresse);
    tmp_ret = adwin_test_version(Linkaddresse);
    if (tmp_ret != 0) return 3;
    else return 0;
}
int16_t e_ADTest_Version (int16_t Linkaddresse, int16_t msgbox, int32_t *error)   // Test_Version
{
    int tmp_ret;
    Set_DeviceNo (Linkaddresse);
    tmp_ret = adwin_test_version(Linkaddresse);
    *error = Get_Last_Error();
    if (tmp_ret != 0) return 3;
    else return 0;
}
// Free_Mem
int32_t AD_Memory_all (int16_t typ, int16_t Linkaddresse)
{
    Set_DeviceNo (Linkaddresse);
    return adwin_mem(typ, Linkaddresse);
}
int32_t e_AD_Memory_all (int16_t typ, int16_t Linkaddresse, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (Linkaddresse);
    tmp = adwin_mem(typ, Linkaddresse);
    *error = Get_Last_Error();
    return tmp;
}
int32_t AD_Memory_all_byte(int16_t typ, int16_t Linkaddresse)
{
    Set_DeviceNo (Linkaddresse);
    return Free_Mem(typ);
}
int32_t e_AD_Memory_all_byte(int16_t typ, int16_t Linkaddresse, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (Linkaddresse);
    tmp = Free_Mem(typ);
    *error = Get_Last_Error();
    return tmp;
}
int32_t AD_Memory(int16_t Linkaddresse)
{
    adwin_errno = k_function_not_allowed;
    return 255;
}
int32_t e_AD_Memory(int16_t Linkaddresse, int32_t *error)
{
    adwin_errno = k_function_not_allowed;
    *error = Get_Last_Error();
    return 255;
}


// ADNet_Connect und / Disconnect bleibt draussen laut SJ am 8.5.03
int16_t ADProzessorTyp(int16_t Linkaddresse)
{
    Set_DeviceNo (Linkaddresse);
    return adwin_processor_type(Linkaddresse);
}
int16_t e_ADProzessorTyp(int16_t Linkaddresse, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (Linkaddresse);
    tmp = adwin_processor_type(Linkaddresse);
    *error = Get_Last_Error();
    return tmp;
}
int16_t e_Clear_Process(int16_t proc_nr, int16_t nadr, int32_t *error)
{
    Set_DeviceNo(nadr);
    Clear_Process(proc_nr);
    *error = Get_Last_Error();
    return (*error ? 255 : 0);
}

int16_t Get_ADBPar_All(int16_t np, int16_t anzahl, int32_t *pData, int16_t nadr) //Get_Par_Block
{
    Set_DeviceNo (nadr);
    return adwin_get_par_block(np, anzahl, (int32_t *) pData, nadr);
}
int16_t e_Get_ADBPar_All(int16_t np, int16_t anzahl, int32_t *pData, int16_t nadr, int32_t *error) //Get_Par_Block
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_get_par_block(np, anzahl, (int32_t *) pData, nadr);
    *error = Get_Last_Error();
    return tmp;
}
int16_t Get_ADBFPar_All(int16_t np, int16_t anzahl, float *pData, int16_t nadr) //Get_FPar_Block
{
    Set_DeviceNo (nadr);
    return adwin_get_fpar_block(np, anzahl, pData, nadr);
}
int16_t e_Get_ADBFPar_All(int16_t np, int16_t anzahl, float *pData, int16_t nadr, int32_t *error) //Get_FPar_Block
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_get_fpar_block(np, anzahl, pData, nadr);
    *error = Get_Last_Error();
    return tmp;
}
int32_t GetDataLength(int16_t nr, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_data_length(nr, nadr);
}
int32_t e_GetDataLength(int16_t nr, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_data_length(nr, nadr);
    *error = Get_Last_Error();
    return tmp;
}


/* vorerst nicht:
int16_t Get_Known_DeviceNo(int32_t *Devices, int32_t Count_Devices);
DLL: int16_t Get_Known_Deviceno(int16_t *devices, int32_t *count_devices);

int32_t Get_Known_USB_SerialNo(int32_t *SerialNo);
DLL: int32_t Get_Known_USB_SerialNo(int32_t *serialno);

int AD_Debug_On(BSTR * Filename, int32_t Size);
DLL: int adwin_debug_mode_on(const char *dateiname, int32_t size);

void AD_Debug_Off();
DLL: int adwin_debug_mode_off();
*/


int32_t Get_Data_String (char *Data, int32_t MaxCount, int16_t DataNo, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_get_data_string(DataNo, MaxCount, Data, nadr);
}
int32_t e_Get_Data_String (char *Data, int32_t MaxCount, int16_t DataNo, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_get_data_string(DataNo, MaxCount, Data, nadr);
    *error = Get_Last_Error();
    return tmp;
}
int32_t Get_Data_String_Length(int32_t DataNo, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_strlen(DataNo, nadr);
}
int32_t e_Get_Data_String_Length(int32_t DataNo, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_strlen(DataNo, nadr);
    *error = Get_Last_Error();
    return tmp;
}
int32_t Set_Data_String(const char *Data, int16_t DataNo, int16_t nadr)
{
    Set_DeviceNo (nadr);
    return adwin_set_data_string(DataNo, Data, nadr);
}
int32_t e_Set_Data_String(const char *Data, int16_t DataNo, int16_t nadr, int32_t *error)
{
    int32_t tmp;

    Set_DeviceNo (nadr);
    tmp = adwin_set_data_string(DataNo, Data, nadr);
    *error = Get_Last_Error();
    return tmp;
}

void Set_Globaldelay(int32_t ProcessNo, int32_t Value)
{
    Set_Processdelay(ProcessNo, Value);
}

int adwin_set_globaldelay(int process, int delay, int id)
{
    return adwin_set_processdelay(process, delay, id);
}

int32_t Get_Globaldelay(int32_t ProcessNo)
{
    return Get_Processdelay(ProcessNo);
}

int adwin_get_globaldelay(int process, int id)
{
    return adwin_get_processdelay(process, id);
}

int __adwin_get_globaldelay(int process, int *err, int id)
{
    return __adwin_get_processdelay(process, err, id);
}

int32_t e_Set_GD_Transsize(int32_t value, int16_t nadr, int32_t* error)
{
    if (Set_GD_Transsize(value, nadr)) {
        *error = Get_Last_Error();
        return -1;
    }
    return 0;
}

int32_t e_Get_GD_Transsize(int16_t nadr, int32_t* error)
{
    int32_t tmp = Get_GD_Transsize(nadr);
    *error = Get_Last_Error();
    return tmp;
}
