#ifndef __LIBADWIN_COMPAT_H__
#define __LIBADWIN_COMPAT_H__

#include <sys/cdefs.h>
#include <sys/types.h>
#include <stdint.h>
#include "libadwin/bits.h"

#ifdef __cplusplus
extern "C" {
#endif
extern long    adwin_boot(const char *btl, long iboardsize, int id);
extern int32_t adwin_process_load (const char *filename, int32_t deviceno);
extern int32_t adwin_process_start (int32_t processno, int32_t deviceno);
extern int32_t adwin_process_stop (int32_t processno, int32_t deviceno);
extern int32_t adwin_process_clear (int32_t processno, int32_t deviceno);
extern int32_t adwin_process_status (int32_t processno, int32_t deviceno);

extern int32_t adwin_set_globaldelay (int32_t processno, int32_t value, int32_t deviceno);
extern int32_t adwin_set_par (int32_t Index, int32_t Value, int32_t deviceno);
extern int32_t adwin_set_fpar (int32_t Index, float Value, int32_t deviceno);

extern int32_t adwin_get_globaldelay (int32_t processno, int32_t deviceno);
extern int32_t adwin_get_par (int32_t par, int32_t deviceno);
extern float adwin_get_fpar (int32_t par, int32_t deviceno);

extern int32_t adwin_get_par_all (int32_t *array, int32_t deviceno);
extern int32_t adwin_get_fpar_all (float *array, int32_t deviceno);
extern int32_t adwin_get_fpar_all_double (double *array, int32_t deviceno);

extern int32_t adwin_get_par_block (int32_t first, int32_t count, int32_t *array, int32_t deviceno);
extern int32_t adwin_get_fpar_block (int32_t first, int32_t count, float *array, int32_t deviceno);
extern int32_t adwin_get_fpar_block_double (int32_t first, int32_t count, double *array, int32_t deviceno);

extern int32_t adwin_get_data_int16 (int32_t datano, int32_t first, int32_t count, int16_t *dest, int32_t deviceno);
extern int32_t adwin_get_data_int32 (int32_t datano, int32_t first, int32_t count, int32_t *dest, int32_t deviceno);
extern int32_t adwin_get_data_int64 (int32_t datano, int32_t first, int32_t count, int64_t *dest, int32_t deviceno);
extern int32_t adwin_get_data_float (int32_t datano, int32_t first, int32_t count, float *dest, int32_t deviceno);
extern int32_t adwin_get_data_double (int32_t datano, int32_t first, int32_t count, double *dest, int32_t deviceno);

extern int32_t adwin_set_data_int16 (int32_t datano, int32_t first, int32_t count, int16_t *src, int32_t deviceno);
extern int32_t adwin_set_data_int32 (int32_t datano, int32_t first, int32_t count, int32_t *src, int32_t deviceno);
extern int32_t adwin_set_data_int64 (int32_t datano, int32_t first, int32_t count, int64_t *src, int32_t deviceno);
extern int32_t adwin_set_data_float (int32_t datano, int32_t first, int32_t count, float *src, int32_t deviceno);
extern int32_t adwin_set_data_double (int32_t datano, int32_t first, int32_t count, double *src, int32_t deviceno);

extern int32_t adwin_clear_data (int32_t datano, int32_t deviceno);

extern int32_t adwin_data_length (int32_t datano, int32_t deviceno);
extern int32_t adwin_data2file (const char *filename, int32_t datano, int32_t first, int32_t count, int32_t mode, int32_t deviceno);
extern int32_t adwin_file2data(const char *filename, adwin_data_type_t data_type, int datano, int32_t first, int id);

extern int32_t adwin_get_data_type(int32_t datano, int32_t deviceno);

extern int32_t adwin_fifo_clear (int32_t fifono, int32_t deviceno);
extern int32_t adwin_fifo_empty (int32_t fifono, int32_t deviceno);
extern int32_t adwin_fifo_full (int32_t fifono, int32_t deviceno);

extern int32_t adwin_set_fifo_int16 (int32_t fifono, int32_t count, int16_t *src, int32_t deviceno);
extern int32_t adwin_set_fifo_int32 (int32_t fifono, int32_t count, int32_t *src, int32_t deviceno);
extern int32_t adwin_set_fifo_int64 (int32_t fifono, int32_t count, int64_t *src, int32_t deviceno);
extern int32_t adwin_set_fifo_float (int32_t fifono, int32_t count, float *src, int32_t deviceno);
extern int32_t adwin_set_fifo_double (int32_t fifono, int32_t count, double *src,  int32_t deviceno);

extern int32_t adwin_get_fifo_int16 (int32_t fifono, int32_t count, int16_t *dest, int32_t deviceno);
extern int32_t adwin_get_fifo_int32 (int32_t fifono, int32_t count, int32_t *dest, int32_t deviceno);
extern int32_t adwin_get_fifo_int64 (int32_t fifono, int32_t count, int64_t *dest, int32_t deviceno);
extern int32_t adwin_get_fifo_float (int32_t fifono, int32_t count, float *dest, int32_t deviceno);
extern int32_t adwin_get_fifo_double (int32_t fifono, int32_t count, double *dest, int32_t deviceno);

extern int32_t adwin_get_data_string (int32_t datano, int32_t count, char *dest, int32_t deviceno);
extern int32_t adwin_set_data_string (int32_t datano, const char *src, int32_t deviceno);
extern int32_t adwin_strlen (int32_t datano, int32_t deviceno);

extern long adwin_mem(mem_spec_t mem_spec, int id);
extern int32_t adwin_get_memory_dsp(int32_t base_addr, int32_t count, int32_t *dest, int32_t deviceno);

extern int32_t adwin_workload(int32_t Priority, int32_t id);
extern int32_t adwin_test_version (int32_t deviceno);
extern processor_type_t adwin_processor_type(int id);


extern char *libadwin_version(void);
/*
 * Linux only !
 * A nonzero @err argument indicates an error.
 * Include "libadwin/errno.h as well and use
 * adwin_strerror(err) (not thread-safe) or
 * adwin_strerror_r(err, *buf, n) (thread-safe)
 * to get an error description.
 */
extern long __adwin_mem(mem_spec_t mem_spec, int *err, int id);

extern int32_t __adwin_get_globaldelay (int32_t processno, int32_t *err, int32_t deviceno);
extern int32_t __adwin_get_par (int32_t index, int32_t *err, int32_t deviceno);
extern float __adwin_get_fpar (int32_t index, int32_t *err, int32_t deviceno);
extern int32_t __adwin_workload(int32_t Priority, int32_t *err, int32_t id);

extern int32_t __adwin_fifo_empty (int32_t fifono, int32_t *err, int32_t deviceno);
extern int32_t __adwin_fifo_full (int32_t fifono, int32_t *err, int32_t deviceno);

extern int32_t __adwin_data_length (int32_t datano, int32_t *err, int32_t deviceno);

// Boot
extern int32_t   ADboot(const char *Dateiname, int16_t Linkadresse,
                        int32_t iboardsize, int16_t msgbox);
extern int32_t e_ADboot(const char *Dateiname, int16_t Linkadresse,
                        int32_t iboardsize, int16_t msgbox, int32_t *error);

extern int32_t ADGetErrorCode(void);
extern int32_t ADGetErrorText(int32_t ErrCode, char *pText, int32_t len);
extern int16_t ADSetLanguage(int16_t Lang);

// Load_Process
extern int16_t ADBload(const char *dateiname, int16_t Linkaddresse, int16_t msgbox);
extern int16_t e_ADBload(const char *dateiname, int16_t Linkaddresse, int16_t msgbox, int32_t *error);

// Start_process
extern int16_t   ADB_Start(int16_t np, int16_t nadr);
extern int16_t e_ADB_Start(int16_t np, int16_t nadr, int32_t *error);

extern int16_t Get_List(int16_t *tarr, int16_t typ, int16_t nadc, int16_t proc, int32_t il, int16_t nadr);
extern int16_t Set_List(int16_t *tarr, int16_t typ, int16_t nadc, int32_t il, int16_t nadr);
extern int16_t Start(int16_t np, int16_t nadr);  //Start von Standardprozessen
extern int16_t Stop(int16_t np, int16_t nadr);  //Stop von Standardprozessen

// Stop_Process
extern int16_t   ADB_Stop(int16_t np, int16_t nadr);
extern int16_t e_ADB_Stop(int16_t np, int16_t nadr, int32_t *error);

// Set_Par
extern int16_t   Set_ADBPar(int16_t np, int32_t iw, int16_t nadr);
extern int16_t e_Set_ADBPar(int16_t np, int32_t iw, int16_t nadr, int32_t *error);

// Set_FPar
extern int16_t   Set_ADBFPar(int16_t np, float fw, int16_t nadr);
extern int16_t e_Set_ADBFPar(int16_t np, float fw, int16_t nadr, int32_t *error);

// Get_Par
extern int32_t   Get_ADBPar(int16_t np, int16_t nadr);
extern int32_t e_Get_ADBPar(int16_t np, int16_t nadr, int32_t *error);

// Get_FPar
extern float   Get_ADBFPar(int16_t np, int16_t nadr);
extern float e_Get_ADBFPar(int16_t np, int16_t nadr, int32_t *error);

// Data2File
extern int16_t   SaveFast(const char *dateiname, int16_t Datanr,
                          int32_t startindex, int32_t anzahldaten,
                          int16_t anhaengen, int16_t linkaddresse);
extern int16_t e_SaveFast(const char *dateiname, int16_t Datanr,
                          int32_t startindex, int32_t anzahldaten,
                          int16_t anhaengen, int16_t linkaddresse,
                          int32_t *error);

// Get_Data
extern int16_t   Get_Data(void *dest, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no);
extern int16_t e_Get_Data(void *dest, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no,
                          int32_t *error);

// Set_Data
extern int16_t   Set_Data(void *src, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no);
extern int16_t e_Set_Data(void *src, adwin_data_type_t type, int16_t data_no,
                          int32_t start_index, int32_t count, int16_t device_no,
                          int32_t *error);

// Get_Fifo
extern int16_t   Get_Fifo(void *tarr, adwin_data_type_t type, int16_t nadc,
                          int32_t il, int16_t nadr);
extern int16_t e_Get_Fifo(void *tarr, adwin_data_type_t type, int16_t nadc,
                          int32_t il, int16_t nadr, int32_t *error);

// Set_Fifo
extern int16_t   Set_Fifo(void *tarr, adwin_data_type_t type, int16_t nadc,
                          int32_t il, int16_t nadr);
extern int16_t e_Set_Fifo(void *tarr, adwin_data_type_t type, int16_t nadc,
                          int32_t il, int16_t nadr, int32_t *error);

// Fifo_Full
extern int32_t   Get_Fifo_Count(int16_t nadc, int16_t nadr );
extern int32_t e_Get_Fifo_Count(int16_t nadc, int16_t nadr, int32_t *error);

// Fifo_Empty
extern int32_t   Get_Fifo_Empty(int16_t nadc, int16_t nadr );
extern int32_t e_Get_Fifo_Empty(int16_t nadc, int16_t nadr, int32_t *error);

//Fifo_Clear
extern int16_t   Clear_Fifo(int16_t nadc, int16_t nadr );
extern int16_t e_Clear_Fifo(int16_t nadc, int16_t nadr, int32_t *error);

uint16_t Get_ADC (int16_t nadc, int16_t nadr); //ADC
int16_t Set_DAC (int16_t ndac, int16_t iw, int16_t nadr);  //SetDAC


extern int32_t Get_Digin(int16_t nadr);  //DigIn
extern int16_t Set_Digout(int16_t iw, int16_t nadr);  //SetDigOut
extern int32_t Get_Digout(int16_t nadr);

//Workload
extern int16_t   AD_Workload(int32_t Priority, int16_t Linkaddresse);
extern int16_t e_AD_Workload(int32_t Priority, int16_t Linkaddresse, int32_t *error);

// Test_Version
extern int16_t   ADTest_Version(int16_t Linkaddresse, int16_t msgbox);
extern int16_t e_ADTest_Version(int16_t Linkaddresse, int16_t msgbox, int32_t *error);

// Free_Mem
extern int32_t   AD_Memory_all(int16_t typ, int16_t Linkaddresse);
extern int32_t e_AD_Memory_all(int16_t typ, int16_t Linkaddresse, int32_t *error);

extern int32_t   AD_Memory(int16_t Linkaddresse);
extern int32_t e_AD_Memory(int16_t Linkaddresse, int32_t *error);

extern int32_t   AD_Memory_all_byte(int16_t typ, int16_t Linkaddresse);
extern int32_t e_AD_Memory_all_byte(int16_t typ, int16_t Linkaddresse, int32_t *error);


extern int16_t   ADProzessorTyp(int16_t Linkaddresse);
extern int16_t e_ADProzessorTyp(int16_t Linkaddresse, int32_t *error);

extern int16_t e_Clear_Process(int16_t proc_nr, int16_t nadr, int32_t *error);

//Get_Par_Block
extern int16_t   Get_ADBPar_All(int16_t np, int16_t anzahl, int32_t *pData,
                                int16_t nadr);
extern int16_t e_Get_ADBPar_All(int16_t np, int16_t anzahl, int32_t *pData,
                                int16_t nadr, int32_t *error);


//Get_FPar_Block
extern int16_t   Get_ADBFPar_All(int16_t np, int16_t anzahl, float *pData,
                                 int16_t nadr);
extern int16_t e_Get_ADBFPar_All(int16_t np, int16_t anzahl, float *pData,
                                 int16_t nadr, int32_t *error);


extern int32_t   GetDataLength(int16_t nr, int16_t nadr);
extern int32_t e_GetDataLength(int16_t nr, int16_t nadr, int32_t *error);



/* vorerst nicht:
int16_t Get_Known_DeviceNo(int32_t *Devices, int32_t Count_Devices);
DLL: int16_t Get_Known_Deviceno(int16_t *devices, int32_t *count_devices);

int32_t Get_Known_USB_SerialNo(int32_t *SerialNo);
DLL: int32_t Get_Known_USB_SerialNo(int32_t *serialno);

int32_t AD_Debug_On(BSTR * Filename, int32_t Size);
DLL: int32_t adwin_debug_mode_on(const char *dateiname, int32_t size);

void AD_Debug_Off();
DLL: int32_t adwin_debug_mode_off();
*/

extern int32_t   Get_Data_String(char *Data, int32_t MaxCount, int16_t DataNo,
                                 int16_t nadr);
extern int32_t e_Get_Data_String(char *Data, int32_t MaxCount, int16_t DataNo,
                                 int16_t nadr, int32_t *error);



extern int32_t   Get_Data_String_Length(int32_t DataNo, int16_t nadr);
extern int32_t e_Get_Data_String_Length(int32_t DataNo, int16_t nadr, int32_t *error);



extern int32_t   Set_Data_String(const char *Data, int16_t DataNo,
                                 int16_t nadr);
extern int32_t e_Set_Data_String(const char *Data, int16_t DataNo,
                                 int16_t nadr, int32_t *error);


extern int16_t Get_Data_packed(void *tarr, int16_t typ, int16_t nadc, int32_t lstart, int32_t il, int16_t nadr);
/*   typ = 1 for int16_t/short
     typ = 2 for int32_t/long
     typ = 5 for float
     typ = 6 for double  */

extern void    Set_Globaldelay(int32_t process_no, int32_t value);
extern int32_t Get_Globaldelay(int32_t process_no);

extern int32_t e_Set_GD_Transsize(int32_t value, int16_t nadr, int32_t* error);
extern int32_t e_Get_GD_Transsize(int16_t nadr, int32_t* error);

#ifdef __cplusplus
}
#endif

#endif /* __LIBADWIN_COMPAT_H__ */
