#!/usr/bin/python
# -*- coding: utf-8 -*-
# bas_dmo7.py

from PyQt4 import QtCore, QtGui, uic
import os, sys, array, math, ADwin
from Ui_dialog import Ui_Form

DEVICENUMBER = 0x1
BTL = "ADwin9.btl"
PROCESS = "Bas_dmo7.t92"

class Dialog(QtGui.QDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.connect(self.ui.btnQuit, QtCore.SIGNAL("clicked()"), self.quit)
        self.connect(self.ui.btnStart, QtCore.SIGNAL("clicked()"), self.start)
        self.connect(self.ui.btnStop, QtCore.SIGNAL("clicked()"), self.stop)
        self.connect(self.ui.leFreq, QtCore.SIGNAL("editingFinished()"), self.frequencyChanged)
        self.connect(self.ui.leAmp, QtCore.SIGNAL("editingFinished()"), self.amplitudeChanged)
        self.connect(self.ui.rbSine, QtCore.SIGNAL("clicked()"), self.waveformChanged)
        self.connect(self.ui.rbTrapezoidal, QtCore.SIGNAL("clicked()"), self.waveformChanged)
        self.connect(self.ui.rbTriange, QtCore.SIGNAL("clicked()"), self.waveformChanged)
        self.connect(self.ui.rbRamp, QtCore.SIGNAL("clicked()"), self.waveformChanged)
        self.connect(self.ui.rbRectangle, QtCore.SIGNAL("clicked()"), self.waveformChanged)
        
        self.adw = ADwin.ADwin(DEVICENUMBER, 1)
        btl = self.adw.ADwindir + BTL
        if sys.platform == 'win32':
            process = self.adw.ADwindir + "ADbasic\\samples_ADwin\\" + PROCESS
        else:
            process = self.adw.ADwindir + "share/examples/samples_ADwin/" + PROCESS
        if QtGui.QMessageBox.question(self, "bas_dmo7", "Do you want to boot the ADwin-System?", \
            QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default, QtGui.QMessageBox.No | QtGui.QMessageBox.Escape) == QtGui.QMessageBox.Yes:
            self.adw.Boot(btl)
        self.adw.Load_Process(process)
        
        self.adw.Set_Par(8, 1)
        self.waveform = array.array("f", list(range(10001)))
        self.waveformChanged()
        self.frequencyChanged()
        self.amplitudeChanged()

    def signalSine(self):
        print("waveform Sine")
        for i in range(10001):
            self.waveform[i] = math.sin(2 * math.pi * i / 10000)

    def signalTrapezoidal(self):
        print("waveform Trapezoidal")
        for i in range(2500):
            self.waveform[i] = 2.0 * i / 2501 - 1
        for i in range(2500, 5000):
            self.waveform[i] = 1
        for i in range(5000, 7500):
            self.waveform[i] = -2.0 * (i - 5000) / 2501 + 1
        for i in range(7500, 10001):
            self.waveform[i] = -1

    def signalTriange(self):
        print("waveform Triange")
        for i in range(5000):
            self.waveform[i] = 2.0 * ((i+1) - 1) / 4999 - 1
            self.waveform[i+5000] = -(self.waveform[i])
        self.waveform[10000] = self.waveform[0]
        
    def signalRamp(self):
        print("waveform Ramp")
        for i in range(10001):
            self.waveform[i] = 2.0 * (i - 1) / (10000 - 1) - 1
        self.waveform[10000] = self.waveform[0]
        
    def signalRectangle(self):
        print("waveform Rectangle")
        for i in range(5000):
            self.waveform[i] = -1
        for i in range(5000, 10001):
            self.waveform[i] = 1

    def frequencyChanged(self):
        try:
            freq = float(self.ui.leFreq.text())
        except:
            return
        if freq > 1000: freq = 1000
        if freq < 0: freq = 0
        self.ui.leFreq.setText(str(freq))
        self.adw.Set_FPar(1, freq)
        print("frequency: %f" % (freq))

    def amplitudeChanged(self):
        try:
            amp = float(self.ui.leAmp.text())
        except:
            return
        if amp > 10: amp = 10
        if amp < 0: amp = 0
        self.ui.leAmp.setText(str(amp))
        self.adw.Set_Par(7, int(amp * 3276.8))
        print("amplitude: %f" % (amp))

    def waveformChanged(self):
        if self.ui.rbSine.isChecked(): self.signalSine()
        if self.ui.rbTrapezoidal.isChecked(): self.signalTrapezoidal()
        if self.ui.rbTriange.isChecked(): self.signalTriange()
        if self.ui.rbRamp.isChecked(): self.signalRamp()
        if self.ui.rbRectangle.isChecked(): self.signalRectangle()
        self.adw.SetData_Float(self.waveform, 2, 1, 10001)

    def quit(self):
        self.adw.Stop_Process(2)
        self.close()

    def start(self):
        self.adw.Start_Process(2)

    def stop(self):
        self.adw.Stop_Process(2)


app = QtGui.QApplication(sys.argv)
dialog = Dialog()
sys.exit(dialog.exec_())
